@@ -330,7 +330,7 @@ event_editor_constructor (GType type,
 	content_area =
 		gtk_dialog_get_content_area (GTK_DIALOG (priv->recur_window));
 
-	priv->recur_page = recurrence_page_new (editor);
+	priv->recur_page = recurrence_page_new (priv->model, editor);
 	page = COMP_EDITOR_PAGE (priv->recur_page);
 	if (!e_shell_get_express_mode (shell)) {
 		gtk_container_add (
@@ -437,6 +437,9 @@ event_editor_constructed (GObject *object)
 		object, "client",
 		priv->model, "client",
 		G_BINDING_SYNC_CREATE);
+
+	if (G_OBJECT_CLASS (event_editor_parent_class)->constructed)
+		G_OBJECT_CLASS (event_editor_parent_class)->constructed (object);
 }
 
 static void
