@@ -558,6 +558,9 @@ alert_constructed (GObject *object)
 
 		button = button->next;
 	}
+
+	if (G_OBJECT_CLASS (e_alert_parent_class)->constructed)
+		G_OBJECT_CLASS (e_alert_parent_class)->constructed (object);
 }
 
 static void
