@@ -30,6 +30,7 @@
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 
+#include "em-inline-filter.h"
 #include "em-stripsig-filter.h"
 #include "em-format-quote.h"
 
@@ -37,7 +38,7 @@ struct _EMFormatQuotePrivate {
 	gint dummy;
 };
 
-static void emfq_builtin_init(EMFormatQuoteClass *efhc);
+static void emfq_builtin_init (EMFormatQuoteClass *efhc);
 
 static gpointer parent_class;
 
@@ -59,28 +60,36 @@ emfq_format_clone (EMFormat *emf,
                    CamelFolder *folder,
                    const gchar *uid,
                    CamelMimeMessage *msg,
-                   EMFormat *src)
+                   EMFormat *src,
+                   GCancellable *cancellable)
 {
 	EMFormatQuote *emfq = (EMFormatQuote *) emf;
 	const EMFormatHandler *handle;
 	GConfClient *gconf;
 
 	/* Chain up to parent's format_clone() method. */
-	EM_FORMAT_CLASS (parent_class)->format_clone (emf, folder, uid, msg, src);
+	EM_FORMAT_CLASS (parent_class)->format_clone (
+		emf, folder, uid, msg, src, cancellable);
 
 	gconf = gconf_client_get_default ();
-	camel_stream_reset(emfq->stream, NULL);
+	camel_stream_reset (emfq->stream, NULL);
 	if (gconf_client_get_bool(gconf, "/apps/evolution/mail/composer/top_signature", NULL))
 		camel_stream_printf (emfq->stream, "<br>\n");
 	g_object_unref (gconf);
 	handle = em_format_find_handler(emf, "x-evolution/message/prefix");
 	if (handle)
-		handle->handler(emf, emfq->stream, (CamelMimePart *)msg, handle, FALSE);
+		handle->handler (
+			emf, emfq->stream,
+			CAMEL_MIME_PART (msg),
+			handle, cancellable, FALSE);
 	handle = em_format_find_handler(emf, "x-evolution/message/rfc822");
 	if (handle)
-		handle->handler(emf, emfq->stream, (CamelMimePart *)msg, handle, FALSE);
+		handle->handler (
+			emf, emfq->stream,
+			CAMEL_MIME_PART (msg),
+			handle, cancellable, FALSE);
 
-	camel_stream_flush(emfq->stream, NULL);
+	camel_stream_flush (emfq->stream, cancellable, NULL);
 
 	g_signal_emit_by_name(emf, "complete");
 }
@@ -96,7 +105,8 @@ emfq_format_error (EMFormat *emf,
 static void
 emfq_format_source (EMFormat *emf,
                     CamelStream *stream,
-                    CamelMimePart *part)
+                    CamelMimePart *part,
+                    GCancellable *cancellable)
 {
 	CamelStream *filtered_stream;
 	CamelMimeFilter *html_filter;
@@ -111,7 +121,8 @@ emfq_format_source (EMFormat *emf,
 	g_object_unref (html_filter);
 
 	em_format_format_text (
-		emf, filtered_stream, CAMEL_DATA_WRAPPER (part));
+		emf, filtered_stream,
+		CAMEL_DATA_WRAPPER (part), cancellable);
 
 	g_object_unref (filtered_stream);
 }
@@ -121,7 +132,8 @@ emfq_format_attachment (EMFormat *emf,
                         CamelStream *stream,
                         CamelMimePart *part,
                         const gchar *mime_type,
-                        const EMFormatHandler *handle)
+                        const EMFormatHandler *handle,
+                        GCancellable *cancellable)
 {
 	EMFormatQuote *emfq = EM_FORMAT_QUOTE (emf);
 	gchar *text, *html;
@@ -131,20 +143,21 @@ emfq_format_attachment (EMFormat *emf,
 
 	camel_stream_write_string (
 		stream, "<table border=1 cellspacing=0 cellpadding=0>"
-		"<tr><td><font size=-1>\n", NULL);
+		"<tr><td><font size=-1>\n", cancellable, NULL);
 
 	/* output some info about it */
 	text = em_format_describe_part (part, mime_type);
 	html = camel_text_to_html (
 		text, emfq->text_html_flags &
 		CAMEL_MIME_FILTER_TOHTML_CONVERT_URLS, 0);
-	camel_stream_write_string (stream, html, NULL);
+	camel_stream_write_string (stream, html, cancellable, NULL);
 	g_free (html);
 	g_free (text);
 
-	camel_stream_write_string (stream, "</font></td></tr></table>", NULL);
+	camel_stream_write_string (
+		stream, "</font></td></tr></table>", cancellable, NULL);
 
-	handle->handler (emf, stream, part, handle, FALSE);
+	handle->handler (emf, stream, part, handle, cancellable, FALSE);
 }
 
 static void
@@ -475,6 +488,7 @@ emfq_format_message_prefix (EMFormat *emf,
                             CamelStream *stream,
                             CamelMimePart *part,
                             const EMFormatHandler *info,
+                            GCancellable *cancellable,
                             gboolean is_fallback)
 {
 	EMFormatQuote *emfq = (EMFormatQuote *) emf;
@@ -488,6 +502,7 @@ emfq_format_message (EMFormat *emf,
                      CamelStream *stream,
                      CamelMimePart *part,
                      const EMFormatHandler *info,
+                     GCancellable *cancellable,
                      gboolean is_fallback)
 {
 	EMFormatQuote *emfq = (EMFormatQuote *) emf;
@@ -506,12 +521,59 @@ emfq_format_message (EMFormat *emf,
 	} else if (emfq->flags & EM_FORMAT_QUOTE_HEADERS)
 		emfq_format_headers (emfq, stream, (CamelMedium *)part);
 
-	em_format_part (emf, stream, part);
+	em_format_part (emf, stream, part, cancellable);
 
 	if (emfq->flags & EM_FORMAT_QUOTE_CITE)
 		camel_stream_write_string (
 			stream, "</blockquote><!--+GtkHTML:"
-			"<DATA class=\"ClueFlow\" clear=\"orig\">-->", NULL);
+			"<DATA class=\"ClueFlow\" clear=\"orig\">-->",
+			cancellable, NULL);
+}
+
+/* Decodes inline encoded parts of 'part'. The returned pointer, if not NULL, should
+   be unreffed with g_object_unref().
+*/
+static CamelMimePart *
+decode_inline_parts (CamelMimePart *part, GCancellable *cancellable)
+{
+	CamelMultipart *mp;
+	CamelStream *null;
+	CamelStream *filtered_stream;
+	EMInlineFilter *inline_filter;
+
+	g_return_val_if_fail (part != NULL, NULL);
+
+	null = camel_stream_null_new ();
+	filtered_stream = camel_stream_filter_new (null);
+	g_object_unref (null);
+
+	inline_filter = em_inline_filter_new (camel_mime_part_get_encoding (part), camel_mime_part_get_content_type (part));
+	camel_stream_filter_add (
+		CAMEL_STREAM_FILTER (filtered_stream),
+		CAMEL_MIME_FILTER (inline_filter));
+	camel_data_wrapper_decode_to_stream_sync (
+		camel_medium_get_content (CAMEL_MEDIUM (part)), (CamelStream *)filtered_stream, cancellable, NULL);
+	camel_stream_close ((CamelStream *)filtered_stream, cancellable, NULL);
+	g_object_unref (filtered_stream);
+
+	if (!em_inline_filter_found_any (inline_filter)) {
+		g_object_unref (inline_filter);
+		return NULL;
+	}
+
+	mp = em_inline_filter_get_multipart (inline_filter);
+
+	g_object_unref (inline_filter);
+
+	if (mp) {
+		part = camel_mime_part_new ();
+		camel_medium_set_content (CAMEL_MEDIUM (part), CAMEL_DATA_WRAPPER (mp));
+		g_object_unref (mp);
+	} else {
+		g_object_ref (part);
+	}
+
+	return part;
 }
 
 static void
@@ -519,12 +581,14 @@ emfq_text_plain (EMFormat *emf,
                  CamelStream *stream,
                  CamelMimePart *part,
                  const EMFormatHandler *info,
+                 GCancellable *cancellable,
                  gboolean is_fallback)
 {
 	EMFormatQuote *emfq = EM_FORMAT_QUOTE (emf);
 	CamelStream *filtered_stream;
 	CamelMimeFilter *html_filter;
 	CamelMimeFilter *sig_strip;
+	CamelMimePart *mp;
 	CamelContentType *type;
 	const gchar *format;
 	guint32 rgb = 0x737373, flags;
@@ -532,10 +596,22 @@ emfq_text_plain (EMFormat *emf,
 	if (!part)
 		return;
 
+	mp = decode_inline_parts (part, cancellable);
+	if (mp) {
+		if (CAMEL_IS_MULTIPART (camel_medium_get_content (CAMEL_MEDIUM (mp)))) {
+			em_format_part (emf, stream, mp, cancellable);
+			g_object_unref (mp);
+
+			return;
+		}
+
+		g_object_unref (mp);
+	}
+
 	flags = emfq->text_html_flags;
 
 	/* Check for RFC 2646 flowed text. */
-	type = camel_mime_part_get_content_type(part);
+	type = camel_mime_part_get_content_type (part);
 	if (camel_content_type_is(type, "text", "plain")
 	    && (format = camel_content_type_param(type, "format"))
 	    && !g_ascii_strcasecmp(format, "flowed"))
@@ -550,16 +626,16 @@ emfq_text_plain (EMFormat *emf,
 		g_object_unref (sig_strip);
 	}
 
-	html_filter = camel_mime_filter_tohtml_new(flags, rgb);
+	html_filter = camel_mime_filter_tohtml_new (flags, rgb);
 	camel_stream_filter_add (
 		CAMEL_STREAM_FILTER (filtered_stream), html_filter);
 	g_object_unref (html_filter);
 
 	em_format_format_text (
 		EM_FORMAT (emfq), filtered_stream,
-		CAMEL_DATA_WRAPPER (part));
+		CAMEL_DATA_WRAPPER (part), cancellable);
 
-	camel_stream_flush (filtered_stream, NULL);
+	camel_stream_flush (filtered_stream, cancellable, NULL);
 	g_object_unref (filtered_stream);
 }
 
@@ -568,6 +644,7 @@ emfq_text_enriched (EMFormat *emf,
                     CamelStream *stream,
                     CamelMimePart *part,
                     const EMFormatHandler *info,
+                    GCancellable *cancellable,
                     gboolean is_fallback)
 {
 	CamelStream *filtered_stream;
@@ -577,10 +654,12 @@ emfq_text_enriched (EMFormat *emf,
 	if (g_strcmp0 (info->mime_type, "text/richtext") == 0) {
 		flags = CAMEL_MIME_FILTER_ENRICHED_IS_RICHTEXT;
 		camel_stream_write_string (
-			stream, "\n<!-- text/richtext -->\n", NULL);
+			stream, "\n<!-- text/richtext -->\n",
+			cancellable, NULL);
 	} else {
 		camel_stream_write_string (
-			stream, "\n<!-- text/enriched -->\n", NULL);
+			stream, "\n<!-- text/enriched -->\n",
+			cancellable, NULL);
 	}
 
 	enriched = camel_mime_filter_enriched_new (flags);
@@ -589,9 +668,9 @@ emfq_text_enriched (EMFormat *emf,
 		CAMEL_STREAM_FILTER (filtered_stream), enriched);
 	g_object_unref (enriched);
 
-	camel_stream_write_string (stream, "<br><hr><br>", NULL);
+	camel_stream_write_string (stream, "<br><hr><br>", cancellable, NULL);
 	em_format_format_text (
-		emf, filtered_stream, CAMEL_DATA_WRAPPER (part));
+		emf, filtered_stream, CAMEL_DATA_WRAPPER (part), cancellable);
 	g_object_unref (filtered_stream);
 }
 
@@ -600,10 +679,13 @@ emfq_text_html (EMFormat *emf,
                 CamelStream *stream,
                 CamelMimePart *part,
                 const EMFormatHandler *info,
+                GCancellable *cancellable,
                 gboolean is_fallback)
 {
-	camel_stream_write_string(stream, "\n<!-- text/html -->\n", NULL);
-	em_format_format_text(emf, stream, (CamelDataWrapper *)part);
+	camel_stream_write_string (
+		stream, "\n<!-- text/html -->\n", cancellable, NULL);
+	em_format_format_text (
+		emf, stream, (CamelDataWrapper *)part, cancellable);
 }
 
 static void
@@ -611,6 +693,7 @@ emfq_ignore (EMFormat *emf,
              CamelStream *stream,
              CamelMimePart *part,
              const EMFormatHandler *info,
+             GCancellable *cancellable,
              gboolean is_fallback)
 {
 	/* NOOP */
