@@ -25,12 +25,10 @@
 #include <glib/gi18n.h>
 
 #include "e-util/e-alert-dialog.h"
-#include "e-util/e-binding.h"
 #include "e-util/gconf-bridge.h"
 #include "widgets/misc/e-paned.h"
 
 #include "calendar/common/authentication.h"
-#include "calendar/gui/calendar-config.h"
 #include "calendar/gui/e-calendar-selector.h"
 #include "calendar/gui/misc.h"
 #include "calendar/gui/dialogs/calendar-setup.h"
@@ -59,6 +57,8 @@ struct _ECalShellSidebarPrivate {
 	 * sometime later we update our default-client property
 	 * which is bound by an EBinding to ECalModel. */
 	ECal *default_client;
+
+	GCancellable *loading_default_client;
 };
 
 enum {
@@ -111,7 +111,6 @@ cal_shell_sidebar_backend_died_cb (ECalShellSidebar *cal_shell_sidebar,
                                    ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
 	EShellSidebar *shell_sidebar;
 	GHashTable *client_table;
 	ESource *source;
@@ -121,7 +120,6 @@ cal_shell_sidebar_backend_died_cb (ECalShellSidebar *cal_shell_sidebar,
 
 	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
 
 	source = e_cal_get_source (client);
 	uid = e_source_peek_uid (source);
@@ -131,8 +129,7 @@ cal_shell_sidebar_backend_died_cb (ECalShellSidebar *cal_shell_sidebar,
 	g_hash_table_remove (client_table, uid);
 	cal_shell_sidebar_emit_status_message (cal_shell_sidebar, NULL);
 
-	e_alert_run_dialog_for_args (
-		GTK_WINDOW (shell_window),
+	e_alert_submit (E_ALERT_SINK (e_shell_view_get_shell_content (shell_view)),
 		"calendar:calendar-crashed", NULL);
 
 	g_object_unref (source);
@@ -144,35 +141,18 @@ cal_shell_sidebar_backend_error_cb (ECalShellSidebar *cal_shell_sidebar,
                                     ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
 	EShellSidebar *shell_sidebar;
-	GtkWidget *dialog;
-	const gchar *uri;
-	gchar *uri_no_passwd;
+	ESourceGroup *source_group;
+	ESource *source;
 
 	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
 
-	uri = e_cal_get_uri (client);
-	uri_no_passwd = get_uri_without_password (uri);
+	source = e_cal_get_source (client);
+	source_group = e_source_peek_group (source);
 
-	/* Translators: This string is displayed in a message dialog when
-	 *              our connection to the calendar service detects an
-	 *              out-of-band error.  The first string is a URI for
-	 *              the source of the error, the second string is the
-	 *              error message. */
-	dialog = gtk_message_dialog_new (
-		GTK_WINDOW (shell_window),
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-		_("Error on %s\n%s"),
-		uri_no_passwd, message);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-
-	g_free (uri_no_passwd);
+	e_alert_submit (E_ALERT_SINK (e_shell_view_get_shell_content (shell_view)),
+		"calendar:backend-error", e_source_group_peek_name (source_group), e_source_peek_name (source), message, NULL);
 }
 
 static void
@@ -181,13 +161,11 @@ cal_shell_sidebar_client_opened_cb (ECalShellSidebar *cal_shell_sidebar,
                                     ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
 	EShellSidebar *shell_sidebar;
 	const gchar *message;
 
 	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
 
 	if (g_error_matches (error, E_CALENDAR_ERROR,
 		E_CALENDAR_STATUS_AUTHENTICATION_FAILED) ||
@@ -208,16 +186,14 @@ cal_shell_sidebar_client_opened_cb (ECalShellSidebar *cal_shell_sidebar,
 			return;
 
 		case E_CALENDAR_STATUS_REPOSITORY_OFFLINE:
-			e_alert_run_dialog_for_args (
-				GTK_WINDOW (shell_window),
+			e_alert_submit (E_ALERT_SINK (e_shell_view_get_shell_content (shell_view)),
 				"calendar:prompt-no-contents-offline-calendar",
 				NULL);
 			/* fall through */
 
 		default:
 			if (error->code != E_CALENDAR_STATUS_REPOSITORY_OFFLINE) {
-				e_alert_run_dialog_for_args (
-					GTK_WINDOW (shell_window),
+				e_alert_submit (E_ALERT_SINK (e_shell_view_get_shell_content (shell_view)),
 					"calendar:failed-open-calendar",
 					error->message, NULL);
 			}
@@ -241,104 +217,103 @@ cal_shell_sidebar_client_opened_cb (ECalShellSidebar *cal_shell_sidebar,
 }
 
 static void
-cal_shell_sidebar_default_opened_cb (ECalShellSidebar *cal_shell_sidebar,
-                                     const GError *error,
-                                     ECal *client)
+cal_shell_sidebar_default_loaded_cb (ESource *source,
+                                     GAsyncResult *result,
+                                     EShellSidebar *shell_sidebar)
 {
+	ECalShellSidebarPrivate *priv;
 	EShellView *shell_view;
-	EShellSidebar *shell_sidebar;
+	ECal *client;
+	GError *error = NULL;
+
+	priv = E_CAL_SHELL_SIDEBAR_GET_PRIVATE (shell_sidebar);
 
-	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
 
-	if (g_error_matches (error, E_CALENDAR_ERROR,
-		E_CALENDAR_STATUS_AUTHENTICATION_FAILED) ||
-	    g_error_matches (error, E_CALENDAR_ERROR,
-		E_CALENDAR_STATUS_AUTHENTICATION_REQUIRED))
-		e_auth_cal_forget_password (client);
+	client = e_load_cal_source_finish (source, result, &error);
 
-	/* Handle errors. */
-	switch (error ? error->code : E_CALENDAR_STATUS_OK) {
-		case E_CALENDAR_STATUS_OK:
-			break;
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_error_free (error);
+		goto exit;
 
-		case E_CALENDAR_STATUS_AUTHENTICATION_FAILED:
-			e_cal_open_async (client, FALSE);
-			return;
-
-		case E_CALENDAR_STATUS_BUSY:
-			return;
-
-		default:
-			e_alert_run_dialog_for_args (
-				GTK_WINDOW (e_shell_view_get_shell_window (shell_view)),
-				"calendar:failed-open-calendar",
-				error->message, NULL);
-
-			e_cal_shell_sidebar_remove_source (
-				cal_shell_sidebar,
-				e_cal_get_source (client));
-			return;
+	} else if (error != NULL) {
+		e_alert_submit (E_ALERT_SINK (e_shell_view_get_shell_content (shell_view)),
+			"calendar:failed-open-calendar",
+			error->message, NULL);
+		g_error_free (error);
+		goto exit;
 	}
 
-	g_assert (error == NULL);
+	g_return_if_fail (E_IS_CAL (client));
 
-	g_signal_handlers_disconnect_matched (
-		client, G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
-		cal_shell_sidebar_default_opened_cb, NULL);
+	if (priv->default_client != NULL)
+		g_object_unref (priv->default_client);
 
-	g_object_notify (G_OBJECT (cal_shell_sidebar), "default-client");
+	priv->default_client = client;
+
+	g_object_notify (G_OBJECT (shell_sidebar), "default-client");
+
+exit:
+	g_object_unref (shell_sidebar);
 }
 
 static void
 cal_shell_sidebar_set_default (ECalShellSidebar *cal_shell_sidebar,
                                ESource *source)
 {
+	ECalShellSidebarPrivate *priv;
 	EShellView *shell_view;
+	EShellWindow *shell_window;
 	EShellContent *shell_content;
 	EShellSidebar *shell_sidebar;
 	ECalShellContent *cal_shell_content;
 	ECalSourceType source_type;
-	GHashTable *client_table;
 	ECalModel *model;
 	ECal *client;
 	icaltimezone *timezone;
 	const gchar *uid;
 
+	priv = cal_shell_sidebar->priv;
 	source_type = E_CAL_SOURCE_TYPE_EVENT;
-	client_table = cal_shell_sidebar->priv->client_table;
-
-	uid = e_source_peek_uid (source);
-	client = g_hash_table_lookup (client_table, uid);
-
-	if (cal_shell_sidebar->priv->default_client != NULL)
-		g_object_unref (cal_shell_sidebar->priv->default_client);
-
-	if (client != NULL)
-		g_object_ref (client);
-	else
-		client = e_auth_new_cal_from_source (source, source_type);
-
-	cal_shell_sidebar->priv->default_client = client;
-	g_return_if_fail (client != NULL);
-
-	g_signal_connect_swapped (
-		client, "cal-opened-ex",
-		G_CALLBACK (cal_shell_sidebar_default_opened_cb),
-		cal_shell_sidebar);
 
 	/* FIXME Sidebar should not be accessing the EShellContent.
 	 *       This probably needs to be moved to ECalShellView. */
 	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
 	shell_content = e_shell_view_get_shell_content (shell_view);
+	shell_window = e_shell_view_get_shell_window (shell_view);
 
 	cal_shell_content = E_CAL_SHELL_CONTENT (shell_content);
 	model = e_cal_shell_content_get_model (cal_shell_content);
 	timezone = e_cal_model_get_timezone (model);
 
-	e_cal_set_default_timezone (client, timezone, NULL);
-	e_cal_open_async (client, FALSE);
+	/* Cancel any unfinished previous request. */
+	if (priv->loading_default_client != NULL) {
+		g_cancellable_cancel (priv->loading_default_client);
+		g_object_unref (priv->loading_default_client);
+		priv->loading_default_client = NULL;
+	}
+
+	uid = e_source_peek_uid (source);
+	client = g_hash_table_lookup (priv->client_table, uid);
+
+	/* If we already have an open connection for
+	 * this UID, we can finish immediately. */
+	if (client != NULL) {
+		if (priv->default_client != NULL)
+			g_object_unref (priv->default_client);
+		priv->default_client = g_object_ref (client);
+		g_object_notify (G_OBJECT (shell_sidebar), "default-client");
+		return;
+	}
+
+	priv->loading_default_client = g_cancellable_new ();
+
+	e_load_cal_source_async (
+		source, source_type, timezone,
+		GTK_WINDOW (shell_window), priv->loading_default_client,
+		(GAsyncReadyCallback) cal_shell_sidebar_default_loaded_cb,
+		g_object_ref (shell_sidebar));
 }
 
 static void
@@ -378,12 +353,19 @@ static void
 cal_shell_sidebar_selection_changed_cb (ECalShellSidebar *cal_shell_sidebar,
                                         ESourceSelector *selector)
 {
+	EShellView *shell_view;
+	EShellBackend *shell_backend;
+	EShellSidebar *shell_sidebar;
 	GSList *list, *iter;
 
 	/* This signal is emitted less frequently than "row-changed",
 	 * especially when the model is being rebuilt.  So we'll take
 	 * it easy on poor GConf. */
 
+	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
+	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
+	shell_backend = e_shell_view_get_shell_backend (shell_view);
+
 	list = e_source_selector_get_selection (selector);
 
 	for (iter = list; iter != NULL; iter = iter->next) {
@@ -393,7 +375,8 @@ cal_shell_sidebar_selection_changed_cb (ECalShellSidebar *cal_shell_sidebar,
 		g_object_unref (source);
 	}
 
-	calendar_config_set_calendars_selected (list);
+	e_cal_shell_backend_set_selected_calendars (
+		E_CAL_SHELL_BACKEND (shell_backend), list);
 
 	g_slist_free (list);
 }
@@ -402,31 +385,12 @@ static void
 cal_shell_sidebar_primary_selection_changed_cb (ECalShellSidebar *cal_shell_sidebar,
                                                 ESourceSelector *selector)
 {
-	EShell *shell;
-	EShellView *shell_view;
-	EShellWindow *shell_window;
-	EShellSidebar *shell_sidebar;
-	EShellSettings *shell_settings;
 	ESource *source;
 
-	/* XXX ESourceSelector needs a "primary-selection-uid" property
-	 *     so we can just bind the property with GConfBridge. */
-
-	source = e_source_selector_peek_primary_selection (selector);
+	source = e_source_selector_get_primary_selection (selector);
 	if (source == NULL)
 		return;
 
-	shell_sidebar = E_SHELL_SIDEBAR (cal_shell_sidebar);
-	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
-
-	shell = e_shell_window_get_shell (shell_window);
-	shell_settings = e_shell_get_shell_settings (shell);
-
-	e_shell_settings_set_string (
-		shell_settings, "cal-primary-calendar",
-		e_source_peek_uid (source));
-
 	cal_shell_sidebar_set_default (cal_shell_sidebar, source);
 }
 
@@ -447,7 +411,6 @@ cal_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 	GSList *list, *iter;
 	GObject *object;
 	const gchar *key;
-	gchar *uid;
 
 	priv = E_CAL_SHELL_SIDEBAR_GET_PRIVATE (shell_sidebar);
 
@@ -473,28 +436,29 @@ cal_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 		G_CALLBACK (cal_shell_sidebar_primary_selection_changed_cb),
 		shell_sidebar);
 
-	source = NULL;
-	uid = e_shell_settings_get_string (
-		shell_settings, "cal-primary-calendar");
-	if (uid != NULL)
-		source = e_source_list_peek_source_by_uid (source_list, uid);
-	if (source == NULL)
-		source = e_source_list_peek_source_any (source_list);
-	if (source != NULL)
-		e_source_selector_set_primary_selection (selector, source);
-	g_free (uid);
+	g_object_bind_property_full (
+		shell_settings, "cal-primary-calendar",
+		selector, "primary-selection",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE,
+		(GBindingTransformFunc) e_binding_transform_uid_to_source,
+		(GBindingTransformFunc) e_binding_transform_source_to_uid,
+		g_object_ref (source_list),
+		(GDestroyNotify) g_object_unref);
+
+	list = e_cal_shell_backend_get_selected_calendars (
+		E_CAL_SHELL_BACKEND (shell_backend));
 
-	list = calendar_config_get_calendars_selected ();
 	for (iter = list; iter != NULL; iter = iter->next) {
-		uid = iter->data;
+		const gchar *uid = iter->data;
+
 		source = e_source_list_peek_source_by_uid (source_list, uid);
-		g_free (uid);
 
-		if (source == NULL)
-			continue;
-
-		e_source_selector_select_source (selector, source);
+		if (source != NULL)
+			e_source_selector_select_source (selector, source);
 	}
+
+	g_slist_foreach (list, (GFunc) g_free, NULL);
 	g_slist_free (list);
 
 	/* Listen for subsequent changes to the selector. */
@@ -572,6 +536,12 @@ cal_shell_sidebar_dispose (GObject *object)
 		priv->default_client = NULL;
 	}
 
+	if (priv->loading_default_client != NULL) {
+		g_cancellable_cancel (priv->loading_default_client);
+		g_object_unref (priv->loading_default_client);
+		priv->loading_default_client = NULL;
+	}
+
 	g_hash_table_remove_all (priv->client_table);
 
 	/* Chain up to parent's dispose() method. */
@@ -659,10 +629,10 @@ cal_shell_sidebar_constructed (GObject *object)
 		GtkWidget *button;
 
 		container = gtk_vbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX(container), widget, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (container), widget, TRUE, TRUE, 0);
 
 		button = gtk_button_new_with_mnemonic (_("_New Calendar..."));
-		gtk_box_pack_start (GTK_BOX(container), button, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (container), button, FALSE, FALSE, 0);
 		g_signal_connect (
 			button, "clicked",
 			G_CALLBACK (new_calendar_clicked), shell_sidebar);
@@ -692,13 +662,15 @@ cal_shell_sidebar_constructed (GObject *object)
 	priv->date_navigator = g_object_ref (widget);
 	gtk_widget_show (widget);
 
-	e_binding_new (
+	g_object_bind_property (
 		shell_settings, "cal-show-week-numbers",
-		calitem, "show-week-numbers");
+		calitem, "show-week-numbers",
+		G_BINDING_SYNC_CREATE);
 
-	e_binding_new (
+	g_object_bind_property (
 		shell_settings, "cal-week-start-day",
-		calitem, "week-start-day");
+		calitem, "week-start-day",
+		G_BINDING_SYNC_CREATE);
 
 	/* Restore widget state from the last session once
 	 * the shell view is fully initialized and visible. */
@@ -721,7 +693,7 @@ cal_shell_sidebar_check_state (EShellSidebar *shell_sidebar)
 
 	cal_shell_sidebar = E_CAL_SHELL_SIDEBAR (shell_sidebar);
 	selector = e_cal_shell_sidebar_get_selector (cal_shell_sidebar);
-	source = e_source_selector_peek_primary_selection (selector);
+	source = e_source_selector_get_primary_selection (selector);
 
 	if (source != NULL) {
 		ECal *client;
