@@ -31,6 +31,7 @@
 
 #include <e-util/e-config.h>
 
+#include <mail/e-mail-folder-utils.h>
 #include <mail/e-mail-local.h>
 #include <mail/e-mail-reader.h>
 #include <mail/e-mail-session.h>
@@ -479,7 +480,7 @@ create_new_message (CamelFolder *folder, const gchar *uid, CamelMimeMessage *mes
 	/* FIXME Pass this in somehow. */
 	shell = e_shell_get_default ();
 
-	folder = e_mail_local_get_folder (E_MAIL_FOLDER_TEMPLATES);
+	folder = e_mail_local_get_folder (E_MAIL_LOCAL_FOLDER_TEMPLATES);
 	template = g_object_get_data (G_OBJECT (action), "template");
 
 	/* The new message we are creating */
@@ -617,6 +618,11 @@ build_template_menus_recurse (GtkUIManager *ui_manager,
 			/* FIXME Not passing a GCancellable or GError here. */
 			template = camel_folder_get_message_sync (
 				folder, uid, NULL, NULL);
+
+			/* FIXME Do something more intelligent with errors. */
+			if (template == NULL)
+				continue;
+
 			g_object_ref (template);
 
 			action_label =
@@ -700,7 +706,7 @@ got_message_draft_cb (EMsgComposer *composer,
 	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (message));
 
 	/* Get the templates folder and all UIDs of the messages there. */
-	folder = e_mail_local_get_folder (E_MAIL_FOLDER_TEMPLATES);
+	folder = e_mail_local_get_folder (E_MAIL_LOCAL_FOLDER_TEMPLATES);
 
 	info = camel_message_info_new (NULL);
 
@@ -710,7 +716,10 @@ got_message_draft_cb (EMsgComposer *composer,
 	camel_message_info_set_flags (
 		info, CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_DRAFT, ~0);
 
-	mail_append_mail (folder, message, info, NULL, composer);
+	/* FIXME No async callback, so... hope for the best? */
+	e_mail_folder_append_message (
+		folder, message, info, G_PRIORITY_DEFAULT,
+		NULL, (GAsyncReadyCallback) NULL, NULL);
 
 	g_object_unref (message);
 }
@@ -778,7 +787,8 @@ update_actions_cb (EShellView *shell_view)
 	/* Now recursively build template submenus in the pop-up menu. */
 
 	store = e_mail_local_get_store ();
-	templates_folder = e_mail_local_get_folder (E_MAIL_FOLDER_TEMPLATES);
+	templates_folder = e_mail_local_get_folder (
+		E_MAIL_LOCAL_FOLDER_TEMPLATES);
 	full_name = camel_folder_get_full_name (templates_folder);
 
 	/* FIXME Not passing a GCancellable or GError here. */
