@@ -65,15 +65,16 @@ struct _EShellPrivate {
 
 	gchar *startup_view;
 
-	guint auto_reconnect	: 1;
-	guint modules_loaded	: 1;
-	guint network_available	: 1;
-	guint online		: 1;
-	guint quit_cancelled	: 1;
-	guint safe_mode		: 1;
-	guint express_mode	: 1;
-	guint meego_mode	: 1;
-	guint small_screen_mode	: 1;
+	guint auto_reconnect		: 1;
+	guint express_mode		: 1;
+	guint meego_mode		: 1;
+	guint modules_loaded		: 1;
+	guint network_available		: 1;
+	guint network_available_locked	: 1;
+	guint online			: 1;
+	guint quit_cancelled		: 1;
+	guint safe_mode			: 1;
+	guint small_screen_mode		: 1;
 };
 
 enum {
@@ -95,7 +96,6 @@ enum {
 	PREPARE_FOR_ONLINE,
 	PREPARE_FOR_QUIT,
 	QUIT_REQUESTED,
-	SEND_RECEIVE,
 	WINDOW_CREATED,
 	WINDOW_DESTROYED,
 	LAST_SIGNAL
@@ -220,7 +220,7 @@ shell_ready_for_offline (EShell *shell,
 	 * a signal without triggering the toggle reference. */
 	g_object_ref (activity);
 
-	e_activity_complete (activity);
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
 
 	g_object_remove_toggle_ref (
 		G_OBJECT (activity), (GToggleNotify)
@@ -240,8 +240,11 @@ shell_prepare_for_offline (EShell *shell)
 	if (shell->priv->preparing_for_line_change != NULL)
 		return;
 
-	shell->priv->preparing_for_line_change =
-		e_activity_new (_("Preparing to go offline..."));
+	shell->priv->preparing_for_line_change = e_activity_new ();
+
+	e_activity_set_text (
+		shell->priv->preparing_for_line_change,
+		_("Preparing to go offline..."));
 
 	g_object_add_toggle_ref (
 		G_OBJECT (shell->priv->preparing_for_line_change),
@@ -270,7 +273,7 @@ shell_ready_for_online (EShell *shell,
 	 * a signal without triggering the toggle reference. */
 	g_object_ref (activity);
 
-	e_activity_complete (activity);
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
 
 	g_object_remove_toggle_ref (
 		G_OBJECT (activity), (GToggleNotify)
@@ -290,8 +293,11 @@ shell_prepare_for_online (EShell *shell)
 	if (shell->priv->preparing_for_line_change != NULL)
 		return;
 
-	shell->priv->preparing_for_line_change =
-		e_activity_new (_("Preparing to go online..."));
+	shell->priv->preparing_for_line_change = e_activity_new ();
+
+	e_activity_set_text (
+		shell->priv->preparing_for_line_change,
+		_("Preparing to go online..."));
 
 	g_object_add_toggle_ref (
 		G_OBJECT (shell->priv->preparing_for_line_change),
@@ -322,7 +328,7 @@ shell_ready_for_quit (EShell *shell,
 	 * a signal without triggering the toggle reference. */
 	g_object_ref (activity);
 
-	e_activity_complete (activity);
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
 
 	g_object_remove_toggle_ref (
 		G_OBJECT (activity), (GToggleNotify)
@@ -349,8 +355,11 @@ shell_prepare_for_quit (EShell *shell)
 	if (shell->priv->preparing_for_quit != NULL)
 		return;
 
-	shell->priv->preparing_for_quit =
-		e_activity_new (_("Preparing to quit..."));
+	shell->priv->preparing_for_quit = e_activity_new ();
+
+	e_activity_set_text (
+		shell->priv->preparing_for_quit,
+		_("Preparing to quit..."));
 
 	g_object_add_toggle_ref (
 		G_OBJECT (shell->priv->preparing_for_quit),
@@ -681,6 +690,9 @@ shell_constructed (GObject *object)
 
 	if (!unique_app_is_running (UNIQUE_APP (object)))
 		e_file_lock_create ();
+
+	if (G_OBJECT_CLASS (e_shell_parent_class)->constructed)
+		G_OBJECT_CLASS (e_shell_parent_class)->constructed (object);
 }
 
 static UniqueResponse
@@ -1098,24 +1110,6 @@ e_shell_class_init (EShellClass *class)
 		G_TYPE_NONE, 1,
 		E_TYPE_SHELL_QUIT_REASON);
 
-	/**
-	 * EShell::send-receive
-	 * @shell: the #EShell which emitted the signal
-	 * @parent: a parent #GtkWindow
-	 *
-	 * Emitted when the user chooses the "Send / Receive" action.
-	 * The parent window can be used for showing transient windows.
-	 **/
-	signals[SEND_RECEIVE] = g_signal_new (
-		"send-receive",
-		G_OBJECT_CLASS_TYPE (object_class),
-		G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-		G_STRUCT_OFFSET (EShellClass, send_receive),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__OBJECT,
-		G_TYPE_NONE, 1,
-		GTK_TYPE_WINDOW);
-
 	/**
 	 * EShell::window-created
 	 * @shell: the #EShell which emitted the signal
@@ -1678,23 +1672,6 @@ e_shell_get_active_window (EShell *shell)
 	return GTK_WINDOW (watched_windows->data);
 }
 
-/**
- * e_shell_send_receive:
- * @shell: an #EShell
- * @parent: the parent #GtkWindow
- *
- * Emits the #EShell::send-receive signal.
- **/
-void
-e_shell_send_receive (EShell *shell,
-                      GtkWindow *parent)
-{
-	g_return_if_fail (E_IS_SHELL (shell));
-	g_return_if_fail (GTK_IS_WINDOW (parent));
-
-	g_signal_emit (shell, signals[SEND_RECEIVE], 0, parent);
-}
-
 /**
  * e_shell_get_express_mode:
  * @shell: an #EShell
@@ -1792,6 +1769,9 @@ e_shell_set_network_available (EShell *shell,
 {
 	g_return_if_fail (E_IS_SHELL (shell));
 
+	if (shell->priv->network_available_locked)
+		return;
+
 	if (network_available == shell->priv->network_available)
 		return;
 
@@ -1811,6 +1791,26 @@ e_shell_set_network_available (EShell *shell,
 	}
 }
 
+/**
+ * e_shell_lock_network_available:
+ * @shell: an #EShell
+ *
+ * Locks the value of #EShell:network-available to %TRUE.  Further
+ * attempts to set the property will be ignored.
+ *
+ * This is used for the --force-online command-line option, which is
+ * intended to override the network availability status as reported
+ * by NetworkManager or other network monitoring software.
+ **/
+void
+e_shell_lock_network_available (EShell *shell)
+{
+	g_return_if_fail (E_IS_SHELL (shell));
+
+	e_shell_set_network_available (shell, TRUE);
+	shell->priv->network_available_locked = TRUE;
+}
+
 /**
  * e_shell_get_online:
  * @shell: an #EShell
